package be.iminds.ilabt.jfed.ui.cli2.instruction;

import com.fasterxml.jackson.annotation.JsonTypeInfo;
import com.fasterxml.jackson.databind.DatabindContext;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.cfg.MapperConfig;
import com.fasterxml.jackson.databind.jsontype.impl.TypeIdResolverBase;

import java.util.Arrays;
import java.util.stream.Collectors;

/**
 * InstructionTypeIdResolver
 */
public class InstructionTypeIdResolver extends TypeIdResolverBase {
    @Override
    public String idFromValue(Object o) {
        return ((Instruction) o).getAction().getNiceName();
    }

    @Override
    public String idFromValueAndType(Object o, Class<?> aClass) {
        return ((Instruction) o).getAction().getNiceName();
    }

    static public Class<? extends Instruction> classFromAction(Instruction.Action action) {
        switch (action) {
            case RUN: return RunInstruction.class;
            case CREATESLICE: return CreateSliceInstruction.class;
            case SLICECREDENTIAL: return CredentialInstruction.class;
            case SLICEINFO: return SliceInfoInstruction.class;
            case SLICEACTION: return SliceActionInstruction.class;
            case MANIFEST: return ManifestInstruction.class;
            case DECRYPTPEM: return null;//DecryptPemInstruction.class;
            case RENEW: return RenewInstruction.class;
            case USERINFO: return UserInfoInstruction.class;
            case POA: return PoaInstruction.class;
            case DELETE: return DeleteInstruction.class;
            default: return null;
        }
    }
    
    @Override
    public JavaType typeFromId(DatabindContext context, String id) {
        MapperConfig<?> config = context.getConfig();
        Instruction.Action action = Instruction.Action.parse(id);
        if (action == null)
            return null;
        Class<? extends Instruction> res = classFromAction(action);
        if (res != null)
            return config.constructType(res);
        return null;
    }

    @Override
    public JsonTypeInfo.Id getMechanism() {
        return JsonTypeInfo.Id.CUSTOM;
    }


    /**
     * Helper method used to get a simple description of all known type ids,
     * for use in error messages.
     */
    @Override
    public String getDescForKnownTypeIds() {
        return "[" +Arrays.stream(Instruction.Action.values())
                .map(Instruction.Action::getNiceName)
                .collect(Collectors.joining(", ")) + "]";
    }
}
